<?php

defined('SE_ABSPATH') || exit;

class SeQTranslate extends SeAbstractExtension
{
    public function __construct()
    {
        register_deactivation_hook(WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $this->getBaseName(), array($this, 'deactivateAddon'));
        add_action('activated_plugin', array($this, 'activatedPlugin'), 10, 2);

        parent::__construct();
    }

    public function getBaseName()
    {
        return defined('QTRANSLATE_FILE') ? plugin_basename(QTRANSLATE_FILE) : '';
    }

    public function getBaseNameExtra()
    {
        return 'woocommerce-qtranslate-x/woocommerce-qtranslate-x.php';
    }

    protected function getFilters()
    {
        return array(
            'se_addon_options',
            'se_prepare_product_data',
            'se_generate_taxonomy_attribute',
            'se_generate_filter_from_attribute',
            'se_get_active_languages',
            'se_get_category_data',
            'se_get_product_term_name',
            'se_get_product_tags',
            'se_get_page_data',
            'delete_option',
            'se_get_header',
            'se_get_engines',
            'se_get_frontend_url',
            'se_get_search_results_page_url',
            'se_prepare_product_meta_data',
        );
    }

    protected function getHooks()
    {
        return array(
            'update_option',
        );
    }

    /**
     * Fires after a plugin has been activated.
     *
     * If a plugin is silently activated (such as during an update),
     * this hook does not fire.
     *
     * @since 2.9.0
     *
     * @param string $plugin       Path to the plugin file relative to the plugins directory.
     * @param bool   $network_wide Whether to enable the plugin for all sites in the network
     *                             or just the current site. Multisite only. Default is false.
     */
    public function activatedPlugin($plugin, $network_wide)
    {
        if ($plugin == $this->getBaseName()) {
            $engines = array();
            $lang_codes = $this->getLangCodesList(true);

            foreach ($lang_codes as $lang_code) {
                if (!ApiSe::getInstance()->checkPrivateKey($lang_code)) {
                    // Engine doesn't exist, register one
                    if (ApiSe::getInstance()->signup($lang_code, false, false) == true) {
                        ApiSe::getInstance()->queueImport($lang_code, false);

                        ApiSe::getInstance()->addAdminNotitice(
                            sprintf(
                                __('Searchanise: New search engine for %s is created. Catalog import has started', 'woocommerce-searchanise'),
                                $lang_code
                            ),
                            'info'
                        );
                    }
                } else {
                    // Engine exists run import
                    ApiSe::getInstance()->queueImport($lang_code);
                }
            }

        } elseif ($plugin == $this->getBaseNameExtra()) {
            // qtranlsate for woocommerce, just run reimport
            ApiSe::getInstance()->queueImport();
        }
    }

    /**
     * Set the deactivation hook for a plugin.
     *
     * @since 2.0.0
     */
    public function deactivateAddon()
    {
        $lang_codes = $this->getLangCodesList(true);
        foreach ($lang_codes as $lang_code) {
            ApiSe::getInstance()->addonStatusRequest(ApiSe::ADDON_STATUS_DISABLED, $lang_code);
        }
    }

    /**
     * Returns active lang codes list
     * 
     * @param boolean $exclude_default_language  If true, list will exclude default WPLANG lang code
     * @return array
     */
    private function getLangCodesList($exclude_default_language = false)
    {
        $lang_codes = array();
        $all_locales = $this->getLocales();
        $enabled_languages = get_option('qtranslate_enabled_languages');
        $wp_lang = get_option('WPLANG');

        foreach ($enabled_languages as $lang) {
            if (
                !isset($all_locales[$lang])
                || ($all_locales[$lang] == $wp_lang && $exclude_default_language == true)
            ) {
                continue;
            }

            $lang_codes[] = $all_locales[$lang];

        }

        return $lang_codes;
    }

    public function isActive()
    {
        return is_plugin_active($this->getBaseName());
    }

    /**
     * Addon options hook
     * 
     * @param array $options
     * @return array
     */
    public function seAddonOptions($options)
    {
        if ($this->isActive()) {
            $options['plugins']['qtranslate'] = get_plugin_data(WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $this->getBaseName());
        }

        if (is_plugin_active($this->getBaseNameExtra())) {
            $options['plugins']['woocommerce-qtranslate'] = get_plugin_data(WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $this->getBaseNameExtra());
        }

        return $options;
    }

    private function convertDataLink(&$data, $name, $lang_code)
    {
        if (empty($name) || !isset($data[$name])) {
            return false;
        }

        $lang = $this->getLanguageByCode($lang_code);
        $converted_url = qtranxf_convertURL($data[$name], $lang, true, true);

        if (!empty($converted_url)) {
            $data[$name] = $converted_url;
        }

        return true;
    }

    private function getLocales()
    {
        global $q_config;
        static $qtranxf_locale = array();

        if (empty($qtranxf_locale)) {
            if (!empty($q_config['locale'])) {
                $qtranxf_locale = $q_config['locale'];
            } else {
                $qtranxf_locale = qtranxf_default_locale();
            }
        }

        return $qtranxf_locale;
    }

    public function getDefaultLanguage()
    {
        global $q_config;

        if (!empty($q_config['default_language'])) {
            return $q_config['default_language'];
        } else {
            $locales = $array_flip($this->getLocales());
            $wp_lang = get_option('WPLANG');

            return !empty($wp_lang) && isset($locales[$wp_lang]) ? $locales[$wp_lang] : '';
        }
    }

    private function getLanguageByCode($lang_code)
    {
        $qtranxf_locale = array_flip($this->getLocales());
        return isset($qtranxf_locale[$lang_code]) ? $qtranxf_locale[$lang_code] : $lang_code;
    }

    private function parseValue($value, $lang_code, $use_default_tranlations = true)
    {
        $lang = $this->getLanguageByCode($lang_code);

        $translated_value = qtranxf_use($lang, $value, false, true);
        if (!empty($value) && empty($translated_value) && $use_default_tranlations == true) {
            // No translation for request lang code, get default translation
            $default_language = $this->getDefaultLanguage();
            $translated_value = qtranxf_use($default_language, $value, false, true);
        }

        return $translated_value;
    }

    public function seGetActiveLanguages($active_languages, $lang_code)
    {
        if (empty($lang_code)) {
            $all_locales = $this->getLocales();
            $enabled_languages = get_option('qtranslate_enabled_languages');

            foreach ($enabled_languages as $lang) {
                if (!isset($all_locales[$lang])) {
                    continue;
                }

                $active_languages[] = $all_locales[$lang];
            }

            $active_languages = array_unique($active_languages);
        }

        return $active_languages;
    }

    public function sePrepareProductData($entry, $product_data, $lang_code)
    {
        $text_data = array(
            'full_description',
            'short_description',
            'summary',
            'title',
            'product_code',
        );

        foreach ($text_data as $id) {
            if (isset($entry[$id])) {
                $entry[$id]['value'] = $this->parseValue($entry[$id]['value'], $lang_code);
            }
    
            if (isset($entry['se_grouped_' . $id])) {
                $entry['se_grouped_' . $id] = $this->parseValue($entry['se_grouped_' . $id]['value'], $lang_code);
            }
        }

        $this->convertDataLink($entry['link'], 'value', $lang_code);

        return $entry;
    }

    public function seGenerateTaxonomyAttribute($attribute_data, $attr, $lang_code)
    {
        $attribute_data['title'] = $this->parseValue($attribute_data['title'], $lang_code);

        if (ApiSe::getInstance()->isResultWidgetEnabled($lang_code)) {
            $variants = array();
            $terms = $attr->get_terms();
            $lang = $this->getLanguageByCode($lang_code);

            foreach ($terms as $term) {
                if (!empty($term->i18n_config['name']['ts'][$lang])) {
                    $variants[] = $term->i18n_config['name']['ts'][$lang];
                } else {
                    $variants[] = $term->name;
                }
            }

            $attribute_data['value'] = $variants;
        }

        return $attribute_data;
    }

    public function seGenerateFilterFromAttribute($filter, $attr, $lang_code)
    {
        $filter['label'] = $this->parseValue($filter['label'], $lang_code);

        return $filter;
    }

    public function seGetCategoryData($category_data, $category, $lang_code)
    {
        $lang = $this->getLanguageByCode($lang_code);

        if (!empty($category->i18n_config['name']['ts'][$lang])) {
            $category_data['title'] = $category->i18n_config['name']['ts'][$lang];
        } else {
            $category_data['title'] = $this->parseValue($category_data['title'], $lang_code);
        }

        if (!empty($category->i18n_config['description']['ts'][$lang])) {
            $category_data['summary'] = $category->i18n_config['description']['ts'][$lang];
        } else {
            $category_data['summary'] = $this->parseValue($category_data['summary'], $lang_code);
        }

        $this->convertDataLink($category_data, 'link', $lang_code);

        return $category_data;
    }

    public function seGetProductTermName($term_name, $term, $lang_code)
    {
        $lang = $this->getLanguageByCode($lang_code);

        if (!empty($term->i18n_config['name']['ts'][$lang])) {
            $term_name = $term->i18n_config['name']['ts'][$lang];
        } else {
            $term_name = $this->parseValue($term->name, $lang_code);
        }

        return $term_name;
    }

    public function seGetProductTags(array $product_tags, $lang_code)
    {
        $lang = $this->getLanguageByCode($lang_code);

        foreach ($product_tags as &$term) {
            if (!empty($term->i18n_config['name']['ts'][$lang])) {
                $term->name = $term->i18n_config['name']['ts'][$lang];
            } else {
                $term->name = $this->parseValue($term->name, $lang_code);
            }
        }

        return $product_tags;
    }

    public function seGetPageData($page_data, $post, $lang_code)
    {
        foreach(array('title', 'summary') as $name) {
            if (isset($page_data[$name])) {
                $page_data[$name] = $this->parseValue($page_data[$name], $lang_code);
            }
        }

        $this->convertDataLink($page_data, 'link', $lang_code);

        return $page_data;
    }

    public function updateOption($option_id, $old_value, $value)
    {
        if ($old_value != $value) {
            if ($option_id == 'qtranslate_enabled_languages') {
                $locales = qtranxf_default_locale();
                $disabled_languages = array_diff($old_value, $value);
                $enabled_languages = array_diff($value, $old_value);

                foreach ($enabled_languages as $lang) {
                    if (isset($locales[$lang])) {
                        $check_private_key = ApiSe::getInstance()->checkPrivateKey($locales[$lang]);

                        if (ApiSe::getInstance()->signup($locales[$lang], false, false) == true) {
                            if (!$check_private_key) {
                                // Engine not exists before, run import
                                ApiSe::getInstance()->queueImport($locales[$lang], false);
                                ApiSe::getInstance()->addAdminNotitice(
                                    sprintf(
                                        __('Searchanise: New search engine for %s is created. Catalog import has started', 'woocommerce-searchanise'),
                                        $locales[$lang]
                                    ),
                                    'info'
                                );
                            } else {
                                // Engine exists but need reindexation for actual data
                                ApiSe::getInstance()->queueImport($locales[$lang]);
                            }
                        }
                    }
                }

                foreach ($disabled_languages as $lang) {
                    if (isset($locales[$lang])) {
                        ApiSe::getInstance()->addonStatusRequest(ApiSe::ADDON_STATUS_DISABLED, $locales[$lang]);
                    }
                }
            }
        }
    }

    public function deleteOption($option)
    {
        if ($option == 'qtranslate_enabled_languages') {
            $all_locales = $this->getLocales();
            $enabled_languages = get_option('qtranslate_enabled_languages');

            foreach ($enabled_languages as $lang) {
                if (!empty($all_locales[$lang]) && get_option('WPLANG') != $all_locales[$lang]) {
                    ApiSe::getInstance()->deleteKeys($all_locales[$lang]);
                }
            }
        }
    }

    public function seGetHeader($header, $lang_code)
    {
        $this->convertDataLink($header['header'], 'id', $lang_code);
        return $header;
    }

    public function seGetEngines($engines, $lang_code)
    {
        foreach ($engines as &$engine) {
            $this->convertDataLink($engine, 'url', $engine['lang_code']);
        }

        return $engines;
    }

    public function seGetFrontendUrl($url, $lang_code, $params)
    {
        $data = array('url' => $url);
        $this->convertDataLink($data, 'url', $lang_code);

        return $data['url'];
    }

    public function seGetSearchResultsPageUrl($url, $lang_code)
    {
        $data = array('url' => $url);
        $this->convertDataLink($data, 'url', $lang_code);

        return $data['url'];
    }

    public function sePrepareProductMetaData($data, $lang_code)
    {
        foreach ($data as $name => &$vals) {
            foreach ($vals as &$v) {
                $v = $this->parseValue($v, $lang_code);
                $v = preg_replace('/^(\(.+\) )/', '', $v);
            }
        }

        return $data;
    }
}

// Loads plugin
add_action('plugins_loaded', function() {
    $GLOBALS['SearchaniseQTranslate'] = new SeQTranslate();
}, 5);
